% IPLM.PRO: Theory about well-formedness and consistency of IPL models.
% Author: Frans Meijs, 28 dec 1998.


s_iplm(S) :-
   see(S),
   s_iplmSentenceList([],[]),
   seen.

s_iplmSentenceList(ListOfConsequentTermsReadSofar,
                       ListOfAntecedentTermsReadSofar) :-
   s_iplmSentence(_,
                     ListOfConsequentTermsReadSofar,
                     ListOfConsequentTermsReadSofar2,
                     ListOfAntecedentTermsReadSofar,
                     ListOfAntecedentTermsReadSofar2),
                                  % Object grammatical part.
   s_iplmSentenceList(ListOfConsequentTermsReadSofar2,
                          ListOfAntecedentTermsReadSofar2),
   !.                             % Solution is sufficient.
s_iplmSentenceList(_,_).         % Failure of s_iplmSentence/1
                                  % implies end of recursion.
                                  % This theory should always evaluate
                                  % to "true" in order to get the input 
                                  % file properly closed. Failure 
                                  % conditions have been diverted into 
                                  % instantiation of diagnostics.

                     
s_iplmSentence(S,
                  ListOfConsequentTermsReadSofar,
                  ListOfConsequentTermsReadSofar2,
                  ListOfAntecedentTermsReadSofar,
                  ListOfAntecedentTermsReadSofar2) :-
   read(S),                       % (Does NOT fail at end_of_file!)
   % write('Debug 1: '),write(S),o_eor,
   ~( endOfFileSymbol(S)),
                                  % Object grammatical part.
   o_WellFormedness(S,
       ListOfConsequentTermsReadSofar,
       ListOfConsequentTermsReadSofar2,
       ListOfAntecedentTermsReadSofar,
       ListOfAntecedentTermsReadSofar2).



o_WellFormedness(Fact,
   ListOfConsequentTermsReadSofar,
   ListOfConsequentTermsReadSofar2,
   ListOfAntecedentTermsReadSofar,
   ListOfAntecedentTermsReadSofar) :-
   fact(Fact),
                                  % Object grammatical part.
   o_Consequent__notReferredBy_listOfAntecedents(
                              Fact,
                              ListOfAntecedentTermsReadSofar),
   list__addedWith_term__is_list(ListOfConsequentTermsReadSofar,
                                Fact,
                                ListOfConsequentTermsReadSofar2),
   !.                             % Solution is sufficient.
o_WellFormedness((contradiction(_) :- Antecedent),
                   ListOfConsequentTermsReadSofar,
                   ListOfConsequentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar2) :-
                                  % Object grammatical part.
   o_chars('Def of contradiction...;'),
   o_eor,
   canonicalCompoundTerm__eq_listOfIplTerms(Antecedent, 
                                             ListOfAntecedentTerms),
   list__appendedWith_list__is_list(
                                ListOfAntecedentTerms,
                                ListOfAntecedentTermsReadSofar,
                                ListOfAntecedentTermsReadSofar2),
   !.                             % Solution is sufficient.
                 % Antecedent-terms for which no defining clause exists
                 % are trapped by modern Prolog interpreters
                 % during the query for contradictions.
                 % Consequently, no explicit check has to be done for
                 % this kind of term.
o_WellFormedness((Consequent :- fail),
                   ListOfConsequentTermsReadSofar,
                   ListOfConsequentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar) :-
   o_chars(['Stub for ',Consequent]),
   o_eor,
   o_Consequent__notReferredBy_listOfAntecedents(
                                  Consequent,
                                  ListOfAntecedentTermsReadSofar),
   !.                             % Solution is sufficient.
o_WellFormedness((Consequent :- Antecedent),
                   ListOfConsequentTermsReadSofar,
                   ListOfConsequentTermsReadSofar2,
                   ListOfAntecedentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar2) :-
   canonicalCompoundTerm__eq_listOfIplTerms(Antecedent, 
                                              ListOfAntecedentTerms),
   implication([Consequent,ListOfAntecedentTerms]),   
                                  % No other infix definitions.
   o_ListOfAntecedents__isReferringInto_listOfConsequents(
                                  ListOfAntecedentTerms,
                                  ListOfConsequentTermsReadSofar),
   o_Consequent__notReferredBy_listOfAntecedents(
                                  Consequent,
                                  ListOfAntecedentTermsReadSofar),
   list__addedWith_term__is_list(ListOfConsequentTermsReadSofar,
                                Consequent,
                                ListOfConsequentTermsReadSofar2),
   list__appendedWith_list__is_list(
                                ListOfAntecedentTerms,
                                ListOfAntecedentTermsReadSofar,
                                ListOfAntecedentTermsReadSofar2),
   !.                             % Solution is sufficient.
o_WellFormedness(X,
                   ListOfConsequentTermsReadSofar,
                   ListOfConsequentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar,
                   ListOfAntecedentTermsReadSofar) :-
   o_exceptionMsg('Sentence',X,'is not an IM sentence'),
   o_eor.                  % (The o_eor is for readability of the log file.)
                         % This theory should always evaluate to "true".
   

fact(Fact) :-
   ~(Fact = (_ :- _) ),
                  % Prolog interpreters do not allow to treat the 
                  % functor :- as an atom, so this test has been chosen.
   Fact =.. [_| ArgList],
   list__eq_flatList(ArgList,FlatArgList),
   groundTerm(FlatArgList),
                                  % Object grammatical part.
   o_chars(['Fact ',Fact]),
   o_eor.



implication([Consequent,[AntecedentTerm]]) :-
   Consequent =.. [ConsequentFunctor| ListOfConsequentArg],
   AntecedentTerm =.. [AntecedentFunctor| ListOfAntecedentArg],
   ListOfConsequentArg == ListOfAntecedentArg,
                                  % Object grammatical part.
   o_chars(['Generalization ',
                 ConsequentFunctor,AntecedentFunctor]),
                 o_eor,
   !.                             % Solution is sufficient.
implication([Consequent,ListOfAntecedentTerms]) :- 
   term__has_varArgs(Consequent,ListOfVar),
   ruleAntecedent__binds_vars(ListOfAntecedentTerms,ListOfVar),
                                  % Object grammatical part.
   o_chars(['Abstraction ',Consequent]),
   o_eor.


ruleAntecedent__binds_vars(CT,ListOfVar) :-
   nonvar(CT), nonvar(ListOfVar),
   ruleAntecedent__has_vars(CT,ListOfListOfVar2),
   list__eq_flatList(ListOfListOfVar2,ListOfVar2),
   list__isSubsetOf_list(ListOfVar,ListOfVar2).
 

ruleAntecedent__has_vars(CT,ListOfListOfVar) :-
   nonvar(CT),
   termAtPlace__in_compound2Term__hasVarsEq_headOfList(
                                              1,
                                              CT,
                                              ListOfListOfVar).


termAtPlace__in_compound2Term__hasVarsEq_headOfList(
                                ArgNr,
                                CT,
                                [ListOfVar| ListOfListOfVar] ) :-
   nonvar(ArgNr), nonvar(CT),
   arg(ArgNr,CT,SimpleTerm),
           % Fails after last argument: Used for terminating recursion.
   term__has_varArgs(SimpleTerm,ListOfVar),
   ArgNr2 is ArgNr + 1,
   termAtPlace__in_compound2Term__hasVarsEq_headOfList(
                                                  ArgNr2,
                                                  CT,
                                                  ListOfListOfVar),
   !.                                         % Solution is sufficient.
termAtPlace__in_compound2Term__hasVarsEq_headOfList(ArgNr,CT,_) :-
   nonvar(ArgNr), nonvar(CT),
   ArgNr > 1.      % Terminates recursion and returns "true" for a WFF.
   



term__has_varArgs(~(Term), ListOfVar) :-
   term__has_varArgs(Term,ListOfVar),
   !.                                        % Solution is sufficient.
term__has_varArgs(exists(Term), ListOfVar) :-
   term__has_varArgs(Term,ListOfVar),
   !.                                        % Solution is sufficient.
term__has_varArgs(setofall(_,Term,_), ListOfVar) :-
   term__has_varArgs(Term,ListOfVar),
   !.                                        % Solution is sufficient.
term__has_varArgs(Term,ListOfVar) :-
   nonvar(Term),
   Term =.. [_| ListOfArg],
   list__eq_flatList(ListOfArg,FlatListOfArg),
   flatList__has_vars(FlatListOfArg,ListOfVar).



canonicalCompoundTerm__eq_listOfIplTerms(CT,LOT) :-
   nonvar(CT),
   CT =.. [_,BCT,CT2],
               % This works with interpreters which decompose
               %  (<basic compound term 1>,<basic compound term 2>)
               %     into
               %  [ (,), <basic compound term 1>, <basic compound term 2> ]
               % and
               %  (<basic compound term>,<non-basic canonical compound term>)
               %     into
               %  [ (,), <basic compound term>, (<non-basic canonical compound term>) ]
   basicCompoundTerm__contains_iplTerm(BCT,IT),
                              % Fails on last term in the decomposition.
   canonicalCompoundTerm__eq_listOfIplTerms(CT2,LOT2),
   list__appendedWith_list__is_list([IT],LOT2,LOT),
   !.                                        % Solution is sufficient.
canonicalCompoundTerm__eq_listOfIplTerms(CCT,[IT]) :-
   nonvar(CCT),
   basicCompoundTerm__contains_iplTerm(CCT,IT),
   !.                                        % Solution is sufficient.
canonicalCompoundTerm__eq_listOfIplTerms(X,[]) :-
   o_exceptionMsg('Term',X,'is irregular and hence ignored').
                                         % Cuts and rubbish are ignored.
      % Workaround:
      % It proved to be impossible to handle cuts as ordinary input atoms,
      % so I could not store them in LOT for further analysis.


basicCompoundTerm__contains_iplTerm( ~(IT), IT) :-
   iplTerm(IT),   
   !.                                        % Solution is sufficient.
basicCompoundTerm__contains_iplTerm( exists(IT), IT) :-
   iplTerm(IT),   
   !.                                        % Solution is sufficient.
basicCompoundTerm__contains_iplTerm( propositionSet__has_cardinalNumber(IT,_), IT) :-
   iplTerm(IT),   
   !.                                        % Solution is sufficient.
basicCompoundTerm__contains_iplTerm( setofall(_,IT,_), IT) :-
   iplTerm(IT),   
   !.                                        % Solution is sufficient.
basicCompoundTerm__contains_iplTerm(IT,IT) :-
   iplTerm(IT).


iplTerm(IT) :-
   nonvar(IT),
   IT =.. [_| ArgList],
   list__eq_flatList(ArgList,FlatArgList),
   identifierList(FlatArgList).


identifierList([X]) :-
   identifier(X),
   !.                                        % Solution is sufficient.
identifierList([H| L]) :-
   identifier(H),
   identifierList(L).


identifier(X) :-
   var(X),
   !.                                        % Solution is sufficient.
identifier(X) :- 
   atomic(X).



o_ListOfAntecedents__isReferringInto_listOfConsequents([],_) :-
   !.                                        % Solution is sufficient.
o_ListOfAntecedents__isReferringInto_listOfConsequents([A| LA],LC) :-
   term__isMemberOf_list__v2(A,LC),
   o_exceptionMsg('Antecedent term',A,'is referring backward'),
   o_eor,
   o_ListOfAntecedents__isReferringInto_listOfConsequents(LA,LC),
   !.                                        % Solution is sufficient.
o_ListOfAntecedents__isReferringInto_listOfConsequents([_| LA],LC) :-
   % ~( term__isMemberOf_list__v2(A,LC) ),   % Done by cut above.
   o_ListOfAntecedents__isReferringInto_listOfConsequents(LA,LC).


o_Consequent__notReferredBy_listOfAntecedents('',_) :-
   !.                                        % Solution is sufficient.
o_Consequent__notReferredBy_listOfAntecedents(C,LA) :-
   % write('Debug 2: '),write(C),o_eor,write(LA),o_eor,
   term__isMemberOf_list__v2(C,LA),
   !.                                        % Solution is sufficient.
o_Consequent__notReferredBy_listOfAntecedents(X,_) :-
   o_exceptionMsg('Consequent',X,'has not been referred before'),
   o_eor.

  
% (The o_eors are for readability of the log file.)

% The lists of antecedentTerms are straightforwardly appended to each
% other (by list__appendedWith_list__is_list/3)
% instead of added (by list__addedWith_term__is_list3) because the only
% duplicate terms are those caused by generalizations involving
% multiple inheritance. The latter is a construct to be avoided, see
% file Convent2.txt


  
%:-consult('flatlist.pro').
%:-consult('list.pro').

