% IPLM_ONT.PRO: IPL model to ontology translator.
% Derived from IPLM.PRO.
% (c) Author: Frans A. Meijs, 2000.

% Precondition: A file that has been successfully verified with IPLM_VER


s_iplm(S) :-
   see(S),
   s_iplmSentenceList,
   seen.

s_iplmSentenceList :-
   s_iplmSentence,
   s_iplmSentenceList,
   !.                             % Solution is sufficient.
s_iplmSentenceList.         % Failure of s_iplmSentence/1
                                  % implies end of recursion.
                                  % This theory should always evaluate
                                  % to "true" in order to get the input 
                                  % file properly closed. Failure 
                                  % conditions have been diverted into 
                                  % instantiation of diagnostics.

                     
s_iplmSentence :-
   read(S),                       % (Does NOT fail at end_of_file!)
   % write('Debug 1: '),write(S),o_eor,
   ~( endOfFileSymbol(S)),
                                  % Object grammatical part.
   o_ontology(S).



o_ontology(Fact) :-
   fact(Fact),
   !.                             % Solution is sufficient.
o_ontology((contradiction(_) :- _)) :-
                                  % Object grammatical part.
      % Contradictions are ignored.
   !.                             % Solution is sufficient.
                 % Antecedent-terms for which no defining clause exists
                 % are trapped by modern Prolog interpreters
                 % during the query for contradictions.
                 % Consequently, no explicit check has to be done for
                 % this kind of term.
o_ontology((_ :- fail)) :-
      % Stubs are ignored
   !.                             % Solution is sufficient.
o_ontology((Consequent :- Antecedent)) :-
   canonicalCompoundTerm__eq_listOfIplTerms(Antecedent, 
                                              ListOfAntecedentTerms),
   implication([Consequent,ListOfAntecedentTerms]),   
                                  % No other infix definitions.
   !.                             % Solution is sufficient.
o_ontology(X) :-
   o_chars('% Unrecognized: '),
   o_chars(X),
   o_eor.
                         % This theory should always evaluate to "true".
   

canonicalCompoundTerm__eq_listOfIplTerms(CT,LOT) :-
   nonvar(CT),
   CT =.. [_,BCT,CT2],
               % This works with interpreters which decompose
               %  (<basic compound term 1>,<basic compound term 2>)
               %     into
               %  [ (,), <basic compound term 1>, <basic compound term 2> ]
               % and
               %  (<basic compound term>,<non-basic canonical compound term>)
               %     into
               %  [ (,), <basic compound term>, (<non-basic canonical compound term>) ]
   iplTerm(BCT),
                              % Fails on last term in the decomposition.
   canonicalCompoundTerm__eq_listOfIplTerms(CT2,LOT2),
   list__appendedWith_list__is_list([BCT],LOT2,LOT),
   !.                                        % Solution is sufficient.
canonicalCompoundTerm__eq_listOfIplTerms(CCT,[CCT]) :-
   iplTerm(CCT),
   !.                                        % Solution is sufficient.
canonicalCompoundTerm__eq_listOfIplTerms(_,[]).
                                         % Cuts and rubbish are ignored.
      % Workaround:
      % It proved to be impossible to handle cuts as ordinary input atoms,
      % so I could not store them in LOT for further analysis.


iplTerm(IT) :-
   nonvar(IT),
   IT =.. [_| _].


fact(Fact) :-
   ~(Fact = (_ :- _) ),
                  % Prolog interpreters do not allow to treat the 
                  % functor :- as an atom, so this test has been chosen.
   Fact =.. [Functor| ArgList],
   list__eq_flatList(ArgList,FlatArgList),
   groundTerm(FlatArgList),
                                  % Object grammatical part.
   o_dyadicPrologProposition('concept',Functor,FlatArgList).


implication([Consequent,ListOfAntecedentTerms]) :-
   Consequent =.. [Functor| ListOfArg],
                                  % Object grammatical part.
   o_dyadicPrologProposition_v2('concept',
                              Functor,
                              ListOfArg
                              ),
   o_chars(' :- '),
   antecedent(ListOfAntecedentTerms).


antecedent([Term|[]]) :-
   term(Term),
   o_prologPropositionDelimiter. 
antecedent([Term| ListOfTerms]) :-
   term(Term),
                                  % Object grammatical part.
   o_chars(','),
   o_eor,
   antecedent(ListOfTerms).


term(T) :-
   T =.. [setofall| [Arg1,Arg2,Arg3]],
   o_chars('setofall'),
   o_chars('('),
   o_chars(Arg1),
   o_chars(','),
   term(Arg2),
   o_chars(','),
   o_chars(Arg3),
   o_chars(')'),
   !.
term(T) :-
   T =.. [propositionSet__has_cardinalNumber| [Arg1,Arg2]],
   o_chars('propositionSet__has_cardinalNumber'),
   o_chars('('),
   term(Arg1),
   o_chars(','),
   o_chars(Arg2),
   o_chars(')'),
   !.
term(T) :-
   T =.. [Functor| [Arg]],
   Arg =.. [_| _],
   o_chars(Functor),
   o_chars('('),
   term(Arg),
   o_chars(')'),
   !.
term(T) :-
   T =.. [Functor| ListOfArg],
   o_dyadicPrologProposition_v2('concept',Functor,ListOfArg).

  
:-consult('$PROLOG_FM/outprolo.pro').  
:-consult('$PROLOG_FM/flatlist.pro').  


