% PROL_FM.PRO: Predicates which make Prolog programs less dependent
% on the type of Prolog interpreter.
% Author: Frans A.Ch. Meijs, 1992.

% The SWIprolog version is the source for other versions.



% These op/3 definitions should stay at the top of the file!
% (They must be effectuated immediately.)

%  :-op(700,xfy,\== ).                           % UNSWprolog.
%  X \== Y :- X /= Y.                              % UNSWprolog.
                                 % Predefined in BIM/C/XSB/SWIprolog.
                                 % (Redefinition into "~=" is rejected
                                 % by the BIM/C/XSB/SWIprolog
                                 % interpreters.)


%   :-op(700,xfy,@< ).                           % UNSWprolog.
%   X @< Y :- X < Y.                               % UNSWprolog.
                                 % Predefined in BIM/C/XSB/SWIprolog.

% End of op/3 definitions.


exists(Goal) :- call(Goal),!.
   % The Prolog interpreter will issue an exception message
   % for call(X) with unbound X. Because this clause may be
   % queried relatively frequently it should be fast, so we
   % do no explicit check.


lemma(L) :-
   nonvar(L),
   call(L),
   asserta( (L :- !) ).
                                 % Ref. STERLING86, section 12.2..3
lemma(L) :-
   var(L),
   o_exceptionMsg('lemma',
                [L],
                'is queried with an unbound arg'),
   fail.


inputChar(C) :-      % Being phased out
   i_char(C).
   
i_char(C) :-
%   ~( eof),readc(C2),                              % BIMprolog.
   get0(A),A \== -1,char__eq_asciiNumeral(C2,A), %C/XSB/SWIprolog
%   ~( eof ),getc(C2),                               % UNSWprolog.
   inputChar__mapsInto_char(C2,C),
                         % During backtracking, get0/getc always fails!
   !.                                          % Solution is sufficient.
i_char(C) :-
   endOfFileSymbol(C).


alphanumericChars(X) :- 
   chars__eq_asciiNumeralList(X,[C| _]),
   alphabeticAsciiNumeral(C).
             % Just the first character is checked for being alphabetic(!)


alphabeticAsciiNumeral(X) :-
   X > 64,
   X < 91,
   !.                                          % Solution is sufficient.
alphabeticAsciiNumeral(X) :-
   X > 96,
   X < 123.
 

chars__eq_charList(C,CL) :-
   nonvar(C),
   chars__eq_asciiNumeralList(C,AL),   
   asciiNumeralList__eq_charList(AL,CL),
   !.                                          % Solution is sufficient.
chars__eq_charList(C,CL) :-
   nonvar(CL),
   asciiNumeralList__eq_charList(AL,CL),
   chars__eq_asciiNumeralList(C,AL),
   !.                                          % Solution is sufficient.
chars__eq_charList(X,Y) :-
   var(X),var(Y),
   o_exceptionMsg('chars__eq_charList',
                [X,Y],
                'is queried with two unbound args').
                                           % BIM/C/XSB/SWIprolog.
                                           % (Predefined in UNSWprolog). 

                                             



chars__eq_asciiNumeralList(C,AL) :-
   chars(C),
   name(C,AL),                                 % Non-XSB
%   atom_chars(C,AL),                           % XSB (name/2 errs on leading 0's)
   !.                                          % Solution is sufficient.
chars__eq_asciiNumeralList(C,AL) :-
   var(C),
   groundTerm(AL),
   name(C,AL),                                 % Non-XSB
%   atom_chars(C,AL),                           % XSB (name/2 errs on leading 0's)
   !.                                          % Solution is sufficient.
chars__eq_asciiNumeralList(X,Y) :-
   var(X),var(Y),
   o_exceptionMsg('chars__eq_asciiNumeralList',
                [X,Y],
                'is queried with two unbound args'),
   fail.
                                           % BIM/C/XSB/SWIprolog.
                                           % (Not needed for UNSWprolog)



asciiNumeralList__eq_charList([A],[C]) :-
   char__eq_asciiNumeral(C,A),
   !.                                          % Solution is sufficient.
asciiNumeralList__eq_charList([A| AL],[C| CL]) :-
   char__eq_asciiNumeral(C,A),
   asciiNumeralList__eq_charList(AL,CL),
   !.                                          % Solution is sufficient.
asciiNumeralList__eq_charList(X,Y) :-
   var(X),var(Y),
   o_exceptionMsg('asciiNumeralList__eq_charList',
                [X,Y],
                'is queried with two unbound args'),
   fail.



inputChar__mapsInto_char(C,' ') :-         % BIM/C/XSB/SWIprolog.
   char__eq_asciiNumeral(C,9),              % BIM/C/XSB/SWIprolog.
%inputChar__mapsInto_char('\t',' ') :-      % UNSWprolog.
   !.
inputChar__mapsInto_char(C,EORS) :-        % BIM/C/XSB/SWIprolog.
   char__eq_asciiNumeral(C,10),             % BIM/C/XSB/SWIprolog.
   endOfRecordSymbol(EORS),
%inputChar__mapsInto_char('\n','EOR') :-    % UNSWprolog.
   !.
inputChar__mapsInto_char(C,C) :-
   char__eq_asciiNumeral(C,N),
   N > 31,
   N < 127,        
   !.
inputChar__mapsInto_char(_,'?').
                  % Input tabbing char is replaced by a space char.
                  % End of record is replaced by an endOfRecordSymbol.
                  % Other non-graphic characters are replaced by a '?'.



char__eq_asciiNumeral(C,A) :-   
   chars(C),var(A),
   name(C,[A]),                               % BIM/C/XSB/SWIprolog.
%   ascii(C,A),                                % UNSWprolog.
   !.                                          % Solution is sufficient.
char__eq_asciiNumeral(C,A) :-
   nonvar(A),
   asciiNumber(A),                            % BIM/C/XSB/SWIprolog.
   name(C,[A]),                               % BIM/C/XSB/SWIprolog. 
%   ascii(C,A),                                % UNSWprolog.
   !.                                          % Solution is sufficient.
char__eq_asciiNumeral(X,Y) :-
   var(X),var(Y),
   o_exceptionMsg('char__eq_asciiNumeral',
                [X,Y],
                'is queried with two unbound args'),
   fail.



%o_term(T) :-                            % XSB version.
%   chars(T),                             % XSB makes writeq('*') fail!
%   o_apostrophe,
%   write(T),
%   o_apostrophe,
%   !.                                         % Solution is sufficient.
o_term(T) :-
   writeq(T).




% :- dynamic(incrementingNatGenerator__has_state/2).   % BIM/XSB version


incrementingNatGenerator__with_modulo__has_value(ID,M,N) :-
   nonvar(ID),                      % For more than one incrementingNatGenerator.
   posnat(M),
   var(N),
   retract(incrementingNatGenerator__has_state(ID,N2)),
   N is (N2 + 1) mod M,
   assertz(incrementingNatGenerator__has_state(ID,N)),
   !.                                           % A single solution is required.
incrementingNatGenerator__with_modulo__has_value(ID,M,N) :-
                                    % Initialization, done when the retract fails.
                                    % (Does not work in XSBProlog, so there an
                                    % explicit initial clause has to be defined)
   nonvar(ID),
   posnat(M),
   var(N),
   assertz(incrementingNatGenerator__has_state(ID,0)),
   N = 0,
   !.                                           % A single solution is required.
incrementingNatGenerator__with_modulo__has_value(ID,M,N) :-
   o_exceptionMsg('incrementingNatGenerator__with_modulo__has_value',
                [ID,M,N],
                'is queried with wrong args'),
   fail.




randomReal(R) :- 
   randomNat(S,D),                           % Cprolog/XSB/SWIprolog.
   R is S/D,                                  % Cprolog/XSB/SWIprolog.
%   randomRat(R),                              % UNSWprolog.
   !.                                          % Solution is sufficient.


randomRat([S,D]) :-
   randomNat(S),
   randomNatDiscr(D).


seed(13).                                          % C/SWIprolog.
% :- dynamic(seed/1).                                % BIM/XSBprolog
% :- assertz(seed(13)).                              % BIM/XSBprolog
                                               % Default init.

randomNat(S) :-                        % Portable version (16 bits int).
   retract( seed(S) ),
   randomNatDiscr(D),
   NewSeed is (125*S+1) mod D,
   assertz( seed(NewSeed) ),           % Efficiency of asserta/assertz
                                       % is implementation dependent,
                                       % so there is no preference here.
                                       % However, ISO prolog has no assert/1.
   !.                                          % Solution is sufficient.



randomNatDiscr(4096).


real__isRoundOf_real(Y,X) :-
  Y is round(X).                                   % SWIprolog version

     
real__plus_real__is_real(R1,R2,R3) :-
%   rat__plus_rat__is_rat(R1,R2,R3).               % UNSWprolog.
   R3 is R1 + R2.                                  % BIM/C/XSB/SWIprolog.

real__minus_real__is_real(R1,R2,R3) :-
%   rat__minus_rat__is_rat(R1,R2,R3).             % UNSWprolog.
   R3 is R1 - R2.                                  % BIM/C/XSB/SWIprolog.


real__times_real__is_real(R1,R2,R3) :-
%   rat__times_rat__is_rat(R1,R2,R3).            % UNSWprolog.
   R3 is R1 * R2.                % BIM/C/XSB/SWIprolog.

real__dividedBy_real__is_real(R1,R2,R3) :-
%   rat__dividedBy_rat__is_rat(R1,R2,R3).       % UNSWprolog.
   R3 is R1 / R2.                             % BIM/C/XSB/SWIprolog.


real__div_real__is_int__aux(N,D,Q) :-
   int__div_int__is_int__aux(N,D,Q).

int__div_int__is_int__aux(N,D,Q) :-
%   Q is N / D.                                    % UNSWprolog.
   Q is N // D.                               % BIM/C/XSB/SWIprolog.



real__le_real(R1,R2) :-
%   rat__le_rat(R1,R2).                           % UNSWprolog.
   R1 =< R2.                                  % BIM/C/XSB/SWIprolog.


real__lt_real(R1,R2) :-
%   rat__lt_rat(R1,R2).                           % UNSWprolog.
   R1 < R2.                                   % BIM/C/XSB/SWIprolog.


prologCputime(Secs) :-
%   Secs = 0.                                       % UNSWprolog (dummy).
%   cputime(T),Secs is T // 1000.                  % BIM/Cprolog.
%   cputime(Secs).                                  % XSBprolog
   statistics(cputime,Secs).                      % SWIprolog.




% :- dynamic(bagofallQueueElement/2).               % BIM/XSBprolog



      % Bagofall; a portable and well-behaving version of Findall
      % (See Bratko, p.176..177),
      % with extension for allowing recursive calls.
%bagofall(X,P,_) :-
%   call(P),
%   functor(P,Functor,Arity),
%   assertz( bagofallQueueElement([Functor,Arity],X) ),
                              % [Functor,Arity] is for distinguishing 
                              % the queue from other bagofallQueues.
%   fail.
%bagofall(_,P,X_list) :-
%   functor(P,Functor,Arity),
%   assertz( bagofallQueueElement([Functor,Arity],
%                                   'bagofallQueueEnd') ),
%   bagofallQueue([Functor,Arity],X_list).
%
%bagofallQueue(Queue_ID,X_list) :-
%   retract( bagofallQueueElement(Queue_ID,X) ),
%   bagofallQueueElement__isHeadOf_list(Queue_ID,X,X_list).
%
%bagofallQueueElement__isHeadOf_list(_,X,X_list) :-
%   X = 'bagofallQueueEnd',X_list = [],
%   !.                                         % Solution is sufficient.
%bagofallQueueElement__isHeadOf_list(Queue_ID,X,X_list) :-
%   X_list = [X| Rest],bagofallQueue(Queue_ID,Rest),
%   !.                                         % Solution is sufficient.

bagofall(X,P,X_LIST) :- findall(X,P,X_LIST).    % Native version
         % The portable version does not work anymore with SWIprolog since it has been made ISO-conformant.
         % XSB does not allow a static predicate invoking a dynamic one,
         % so the native one has to be used instead of the portable version.


groundTerm(GT) :- ground(GT).          % SWIprolog/XSB version


term__isMemberOf_list(M,L) :-
   nonvar(L),
   L = [M| _].                   % No cut; it should find ALL solutions.
term__isMemberOf_list(M,L) :-
   nonvar(L),
   L = [_| L2],
   term__isMemberOf_list(M,L2).
term__isMemberOf_list(X,Y) :-
   var(Y),
   o_exceptionMsg('term__isMemberOf_list',
                [X,Y],
                'is queried with unbound second arg'),
   fail.
      % This is a ruggedized version of the standard Member/2 predicate.



term__isMemberOf_list__v2(M,L) :-
   nonvar(L),
   L = [M2| _],
   ~(M2 \= M),
   !.                                          % Solution is sufficient.
term__isMemberOf_list__v2(M,L) :-
   nonvar(L),
   L = [_| L2],
   term__isMemberOf_list__v2(M,L2).
term__isMemberOf_list__v2(X,Y) :-
   var(Y),
   o_exceptionMsg('term__isMemberOf_list__v2',
                [X,Y],
                'is queried with unbound second arg'),
   fail.
      % This version does not bind a variable in M,
      % and it finds only one solution.


% Auxiliary theory for list__eq_flatList/2 in LIST.PRO,
% making it possible to use optimized - but interpreter-specific - predicates.
% Empty sublists are removed, variables are left unchanged.

%list__eq_flatList__aux(L,[]) :-
%   L == [],                    % Variables should NOT be instantiated.
%   !.                                      % Solution is sufficient.
%list__eq_flatList__aux(L,FL) :-
%   L = [L2],
%   list(L2),
%   list__eq_flatList__aux(L2,FL),
%   !.                                      % Solution is sufficient.
%list__eq_flatList__aux(L,FL) :-
%   ~( list(L) ),
%   FL = [L],
%   !.                                      % Solution is sufficient.
%list__eq_flatList__aux([H|T],FL) :-
%   list__eq_flatList__aux(H,FH),
%   list__eq_flatList__aux(T,FT),
%   list__appendedWith_list__is_list__aux(FH,FT,FL).
                                            % Portable, non-optimized version.
                                          
list__eq_flatList__aux(L,FL) :-
   flatten(L,FL).
      % SWIProlog version (built-in).
      % The XSB library clause flatten/2 is not intended for lists(!)
      

%list__eq_flatList__aux(L,FL) :-
%   list__eq_flatList__aux2(L,FL,[]).
%
%list__eq_flatList__aux2([],[],[]) :-
%   !.                                      % Solution is sufficient.
%list__eq_flatList__aux2([H|L],FL,Stack) :-
%   list(H),
%   list__eq_flatList__aux2(H,FL,[L|Stack]),
%   !.                                      % Solution is sufficient.
%list__eq_flatList__aux2([H|L],[H|FL],Stack) :-
%   ~(list(H)),
%   list__eq_flatList__aux2(L,FL,Stack),
%   !.                                      % Solution is sufficient.
%list__eq_flatList__aux2([],FL,[L|Stack]) :-
%   list__eq_flatList__aux2(L,FL,Stack).
   % Optimized code, using a stack for getting rid of the second
   % recursion implied by a non-built-in list__appendedWith_list__is_list/3.
   % Ref. The Art of Prolog, Sterling & Shapiro, MIT Press 1986,
   % section 9.1.
   % However, this implementation causes early stack overflow in XSB1.8



% Auxiliary theory for list__appendedWith_list__is_list/3 in LIST.PRO,
% making it possible to use optimized - but interpreter-specific - predicates.

%list__appendedWith_list__is_list__aux([],L,L) :-
%   !.                                      % Solution is sufficient.
%list__appendedWith_list__is_list__aux([X|L1],L2,[X|L3]) :-
%   list__appendedWith_list__is_list__aux(L1,L2,L3).
      % The classical portable version.
      % (The XSB equivalent is a library-stored one so this can be used instead,
      % for XSB.)
      
list__appendedWith_list__is_list__aux(L1,L2,L3) :-
   append(L1,L2,L3).
      % SWIprolog version (built-in).



%list__withDuplicateMembersRemovedIs_list__aux(L,[]) :-
%   L == [],
%   !.                                         % Solution is sufficient.
%list__withDuplicateMembersRemovedIs_list__aux(L,L2) :-
%   nonvar(L),
%   L = [H| L1],
%   term__isMemberOf_list__v2(H,L1),
%   list__withDuplicateMembersRemovedIs_list__aux(L1,L2),
%   !.                                         % Solution is sufficient.
%list__withDuplicateMembersRemovedIs_list__aux(L,[H|L2]) :-
%   nonvar(L),
%   L = [H| L1],
%   list__withDuplicateMembersRemovedIs_list__aux(L1,L2).
      % The portable version. (Should be replaced by a sort removing
      % dups TBD)

list__withDuplicateMembersRemovedIs_list__aux(L1,L2) :-
   sort(L1,L2).
      % SWIprolog/XSB version (built-in).



arityOfVars__gt_0(X,_) :- var(X),!.
arityOfVars__gt_0(_,Y) :- var(Y).

arityOfVars__gt_1(X,Y) :- var(X),var(Y).

arityOfVars__gt_1(X,Y,_) :- var(X),var(Y),!.
arityOfVars__gt_1(_,Y,Z) :- var(Y),var(Z),!.
arityOfVars__gt_1(X,_,Z) :- var(Z),var(X).

arityOfVars__gt_2(X,Y,Z) :- var(X),var(Y),var(Z).


%call(X) :- X.                                     % UNSWprolog.
                                      % Predefined in BIM/C/XSB/SWIprolog.


%number(X) :- integer(X); rat(X).                  % UNSWprolog.
                                      % Predefined in BIM/C/XSB/SWIprolog.

asciiNumber(X) :- nat(X),X < 128.


o_charsRHSaligned__has_width(C,W) :-
   chars(C),
   name(C,AL),
   o_asciiListRHSaligned__has_width(AL,W),
   !.                   % SWIprolog instantiates AL twice (!)
o_charsRHSaligned__has_width(_,_).


o_natRHSaligned__has_width(N,W) :-
   o_intRHSaligned__has_width(N,W).


o_currencyValue(CV) :-
   o_intRHSaligned__has_width(CV,10).


o_intRHSaligned__has_width(I,W) :-
   integer(I),
   name(I,AL),
   o_asciiListRHSaligned__has_width(AL,W),
   !.                   % Patch: SWIprolog instantiates AL twice(!)
o_intRHSaligned__has_width(_,_).


o_asciiListRHSaligned__has_width(AL,W) :-
   list(AL),
   posnat(W),
   list__has_length(AL,L),
   LeftHandPadding is W - L,
   o_spaces(LeftHandPadding),
   o_asciiList(AL).
o_asciiListRHSaligned__has_width(_,_).


o_charsLHSaligned__has_width(C,W) :-
   chars(C),
   name(C,AL),
   o_asciiListLHSaligned__has_width(AL,W),
   !.                   % Patch: SWIprolog instantiates AL twice(!)
o_charsLHSaligned__has_width(_,_).


o_natLHSaligned__has_width(I,W) :-
   nat(I),
   name(I,AL),
   o_asciiListLHSaligned__has_width(AL,W),
   !.                   % Patch: SWIprolog instantiates AL twice(!)
o_natLHSaligned__has_width(_,_).


o_intLHSaligned__has_width(I,W) :-
   integer(I),
   int__lt_int(I,0),
   name(I,AL),
   o_asciiListLHSaligned__has_width(AL,W),
   !.                   % Patch: SWIprolog instantiates AL twice(!)
o_intLHSaligned__has_width(I,W) :-
   integer(I),
   ~( int__lt_int(I,0) ),
   W2 is W - 1,
   name(I,AL),
   o_spaces(1),                   % space, for suppressed "+" sign.
   o_asciiListLHSaligned__has_width(AL,W2),
   !.                   % Patch: SWIprolog instantiates AL twice(!)
o_intLHSaligned__has_width(_,_).


o_asciiListLHSaligned__has_width(AL,W) :-
   list(AL),
   posnat(W),
   list__has_length(AL,L),
   RightHandPadding is W - L,
   o_asciiList(AL),
   o_spaces(RightHandPadding).
o_asciiListLHSaligned__has_width(_,_).


o_asciiList([]) :-
   !.                                           % Solution is sufficient.
o_asciiList([H|L]) :-
   put(H),
   o_asciiList(L).


list__has_length([],0) :-
   !.                                           % Solution is sufficient.
list__has_length(L,N) :-
   nonvar(L),                    % (To be disabled for unsafe version.)
   nonvar(N),
   L = [_| L2],
   N_minus_1 is N - 1,
   list__has_length(L2,N_minus_1),
   !.                                         % Solution is sufficient.
list__has_length(L,N) :-
   nonvar(L),                    % (To be disabled for unsafe version.)
   var(N),
   L = [_| L2],
   list__has_length(L2,N_minus_1),
   N is N_minus_1 + 1,
   !.                                         % Solution is sufficient.
list__has_length(X,Y) :-
   ~( list(X) ),
   o_exceptionMsg('list__has_length',
                [X,Y],
                'is queried with wrong first arg'),
   fail.
                                   % Length is counted in terms of
                                   % member terms.
                                   % Note: list__has_length([[]],1)
                                   % is true!


o_exceptionMsg(Functor,Arglist,Message) :- 
   exceptionMsgHead(H),
   write(H),
   write(Functor),
   o_spaces(1),
   write(Arglist),
   o_spaces(2),
   write(Message),
   write(';'),
   o_eor.


o_tab(N) :-
   current_output(CO),                       % SWIProlog version.
   line_position(CO,CurrentPos),             % SWIProlog version.
%   CurrentPos = 0,                           % XSB version, TBD
   Offset is N - CurrentPos,
   Offset > 0,
   o_spaces(Offset),
   !.                                        % Solution is sufficient.
o_tab(_).                                    % I/O should never fail.
                  % Puts cursor on position N.
                  % No operation when N <= current position.

                  
o_spaces(N) :- 
   int__le_int(N,0),
   !.                         % Solution is sufficient.
o_spaces(N) :-
   o_char(' '),
   N2 is N - 1,
   o_spaces(N2).
   % Instantiation of N spaces in the output. (Portable version)
   % Replaces tab/1, which is no tab, and does not treat N <= 0 properly.


o_chars(C) :-
   write(C).


o_char(C) :-
   endOfRecordSymbol(C),
   o_eor,
   !.
o_char(C) :-
   write(C).


o_apostrophe :-
   put(39).                                          % Portable version.

o_eor :-                                             % End of output record.
   nl.                                               % Portable version.


o_eop :-                                             % End of printer page.
   put(12).                                          % Portable version.


list(L) :-
   L == [],                               % L should not unify with [].
   !.                                     % Solution is sufficient.
list(L) :-
   nonvar(L),
   L =.. [.|_].



int__le_int(I1,I2) :-
%   I1 <= I2.                                      % UNSWprolog.
   I1 =< I2.                                  % BIM/C/XSB/SWIprolog.
                           % (Redefinition of "=<" into "<=" is rejected
                           % by the Cprolog interpreter.)

int__lt_int(I1,I2) :-             % For consistency with int__le_int().
   I1 < I2.


nat(N) :- integer(N),N >= 0.
   % As "integer" is a pre-specified class of data in Prolog
   % interpreters, the class of natural numerals is expressed in terms
   % of integer numerals, rather than an integer numeral being a set of
   % a sign and a natural numeral.

posnat(P) :- integer(P),P > 0.


chars(X) :- 
   atomic(X),                                     % atomic([]) is true!
   X \== [].


~(X) :-
%   not(X).                                   % C/XSB/UNSWprolog.
   \+ X.                                     % BIM/SWIprolog.



endOfRecordSymbol('EOR').

endOfFileSymbol('end_of_file').    % Also instantiated by read/1 etc.

exceptionMsgHead('!!!EXCEPTION: '). 


% Synonyms for host operating system commands.
d :- ls.
dd(DIR) :- cd(DIR),ls.

