% bitlist.pro: Abstractions about bit lists.


nat__eq_bitList__with_length(N, BL, L) :-
   nonvar(N), nonvar(L),
   nat__eq_bitList(N, BL2),
   list__has_length(BL2, L2),
   L3 is L - L2,
   bitList__paddedWith_bitCount__is_bitList(BL2, L3, BL),
   !.                               % Solution is sufficient.
nat__eq_bitList__with_length(N, BL, L) :-
   o_exceptionMsg('nat__eq_bitList__with_length',
                [N, BL, L],
                'is queried with wrong arguments'),
   fail.
  

bitList__paddedWith_bitCount__is_bitList(BL, 0, BL) :-
   bitList(BL),
   !.                         % Solution is sufficient.
bitList__paddedWith_bitCount__is_bitList(BL, N, BL2) :-
   bitList(BL),
   N > 0,
   bitListPadding__has_length(BLP, N),
   list__appendedWith_list__is_list(BLP, BL, BL2),
   !.                               % Solution is sufficient.
bitList__paddedWith_bitCount__is_bitList(BL, N, BL2) :-
   o_exceptionMsg('bitList__paddedWith_bitCount__is_bitList',
                [BL, N, BL2],
                'is queried with wrong arguments'),
   fail.


bitListPadding__has_length(['0'], 1) :-
   !.                         % Solution is sufficient.
bitListPadding__has_length(BLP, L) :-
   L > 1,
   BLP = ['0'| BLP2],
   L2 is L - 1,
   bitListPadding__has_length(BLP2, L2),
   !.                               % Solution is sufficient.
bitListPadding__has_length(BLP, L) :-
   o_exceptionMsg('bitListPadding__has_length',
                [BLP, L],
                'is queried with wrong arguments'),
   fail.
   
   

nat__eq_bitList(0, ['0']) :-
   !.                         % Solution is sufficient.
nat__eq_bitList(1, ['1']) :-
   !.                         % Solution is sufficient.
nat__eq_bitList(N, BL) :-
   N > 1,
   0 is N mod 2,              % Test on least significant bit.
   nat__withLSBreplacedBy_bitList__eq_bitList(N, ['0'], BL),
   !.                         % Solution is sufficient.
nat__eq_bitList(N, BL) :-
   N > 1,
   nat__withLSBreplacedBy_bitList__eq_bitList(N, ['1'], BL),
   !.                               % Solution is sufficient.
nat__eq_bitList(N, BL) :-
   o_exceptionMsg('nat__eq_bitList',
                [N, BL],
                'is queried with wrong arguments'),
   fail.



nat__withLSBreplacedBy_bitList__eq_bitList(N, BL1, BL2) :-
   nat(N),
   M is N >> 1,
   nat__eq_bitList(M, BL3),
   list__appendedWith_list__is_list(BL3, BL1, BL2),
   !.                               % Solution is sufficient.
nat__withLSBreplacedBy_bitList__eq_bitList(N, BL1, BL2) :-
   o_exceptionMsg('nat__withLSBreplacedBy_bitList__eq_bitList',
                [N, BL1, BL2],
                'is queried with wrong arguments'),
   fail.



bitList(BL) :-
   list__has_length(BL, L),
   L > 0,
   ~( nonBit__isMemberOf_bitList(_, BL) ).


nonBit__isMemberOf_bitList(NB, BL) :-
   term__isMemberOf_list(NB, BL),
   NB \== '0',
   NB \== '1'.


