% CHARS.PRO: Standard theories about character strings.
% Author: Frans Meijs, 1992.

chars__eq_undelimitedChars(C1,C2) :-
   nonvar(C1),
   chars__appendedWith_chars__is_chars(C3,')',C1),
   chars__appendedWith_chars__is_chars('(',C2,C3),
   !.                                          % Solution is sufficient.
chars__eq_undelimitedChars(C,C) :-
   chars(C).
   

chars__appendedWith_chars__is_chars(C1,C2,C3) :-
   chars(C1), chars(C2),
   chars__eq_asciiNumeralList(C1,L1),
   chars__eq_asciiNumeralList(C2,L2),
   list__appendedWith_list__is_list(L1,L2,L3),
   chars__eq_asciiNumeralList(C3,L3),
   !.                                          % Solution is sufficient.
chars__appendedWith_chars__is_chars(C1,C2,C3) :-
   chars(C1), chars(C3),
   chars__eq_asciiNumeralList(C1,L1),
   chars__eq_asciiNumeralList(C3,L3),
   list__appendedWith_list__is_list(L1,L2,L3),
   chars__eq_asciiNumeralList(C2,L2),
   !.                                          % Solution is sufficient.
chars__appendedWith_chars__is_chars(C1,C2,C3) :-
   chars(C2), chars(C3),
   chars__eq_asciiNumeralList(C2,L2),
   chars__eq_asciiNumeralList(C3,L3),
   list__appendedWith_list__is_list(L1,L2,L3),
   chars__eq_asciiNumeralList(C1,L1),
   !.                                          % Solution is sufficient.
chars__appendedWith_chars__is_chars(X,Y,Z) :-
   arityOfVars__gt_1(X,Y,Z),
   o_exceptionMsg('chars__appendedWith_chars__is_chars',
                [X,Y,Z],
                'is queried with more than one unbound arg'),
   fail.


chars__isLexicographicallySubordinateTo_chars(Id1,Id2) :-
   chars(Id1),
   chars(Id2),
   (Id1 @< Id2),
   !.                                      % Solution is sufficient.
chars__isLexicographicallySubordinateTo_chars(X,Y) :-
   var(X), var(Y),
   o_exceptionMsg('chars__isLexicographicallySubordinateTo_chars',
                [X,Y],
                'is queried with unbound args'),
   fail.


chars__eq_flatList(C,[H| T]) :- 
   nonvar(H), var(C),    
   chars__eq_charList(H,HL),
   list__beginsWith_chars(T,T2),
   chars__eq_charList(T2,TL),
   list__appendedWith_list__is_list(HL,TL,CL),
   chars__eq_charList(C,CL),
   !.                                      % Solution is sufficient.
chars__eq_flatList(C,CL) :- 
   nonvar(C),     
   chars__eq_charList(C,CL),
   !.                                      % Solution is sufficient.
chars__eq_flatList(X,Y) :-
   var(X), var(Y),
   o_exceptionMsg('chars__eq_flatList',
                [X,Y],
                'is queried with unbound args'),
   fail.


% :- consult('flatlist.pro').
% :- consult('list.pro').
