% CHARSLST.PRO: Standard theories about lists of character strings.
% Author: Frans Meijs, sep 1993.



chars__isMemberOf_list(CS,L) :-
   chars(CS),
   term__isMemberOf_list(CS,L),
   !.                                      % Solution is sufficient.
chars__isMemberOf_list(X,Y) :-
   var(Y),
   o_exceptionMsg('chars__isMemberOf_list',
              [X,Y],
              'is queried with unbound second arg'),
   fail.


chars__isMemberOf_list__at_location(W,[W| _],1) :-
   chars(W),
   !.                                      % Solution is sufficient.
chars__isMemberOf_list__at_location(W,[_| L],N) :-
   nonvar(L),
   chars__isMemberOf_list__at_location(W,L,N2),
   N is N2 + 1,
   !.                                      % Solution is sufficient.
chars__isMemberOf_list__at_location(X,Y,Z) :-
   o_exceptionMsg('chars__isMemberOf_list__at_location',
              [X,Y,Z],
              'is queried with wrong args, or with unbound second arg'),
   fail.



list__appendedWith_chars__is_list(L1,C,L2) :-
   chars(C),
   list__appendedWith_list__is_list(L1,[C],L2).



list__sortsLexicographicallyInto_list([],[]).
                                      % Base: The empty list is sorted.

list__sortsLexicographicallyInto_list([H| T],Result) :-
   listElement__splits_listInto_list__and_list(H,T,U1,U2),
                                     % U1 = list of entries <= H
                                     % U2 = list of entries > H
   list__sortsLexicographicallyInto_list(U1,V1),
   list__sortsLexicographicallyInto_list(U2,V2),
   list__appendedWith_list__is_list(V1,[H| V2],Result).
                                    % The Quicksort method is specified.


% listElement__splits_listInto_list__and_list(arg1,arg2,arg3,arg4).
% Arg 3 consists of all elements from arg 2 <= arg 1; arg 4 contains
% the remaining elements.

listElement__splits_listInto_list__and_list(_,[],[],[]).
listElement__splits_listInto_list__and_list(
                                          X,[H1| T1],[H1| LT],GE) :-
   id__lt_id(H1,X),
   listElement__splits_listInto_list__and_list(X,T1,LT,GE).
listElement__splits_listInto_list__and_list(
                                          X,[H1| T1],LT,[H1| GE]) :-
   ~(id__lt_id(H1,X)),
   listElement__splits_listInto_list__and_list(X,T1,LT,GE).



id__lt_id(Id1,Id2) :-          
   chars__isLexicographicallySubordinateTo_chars(Id1,Id2),
   !.                                       % Solution is sufficient.
id__lt_id(Id1,Id2) :-
   chars__isLexicographicallySubordinateTo_list(Id1,Id2),
   !.                                       % Solution is sufficient.
id__lt_id(Id1,Id2) :-
   list__isLexicographicallySubordinateTo_list(Id1,Id2).



chars__isLexicographicallySubordinateTo_list(W,L) :-
   chars(W),
   list(L).


list__isLexicographicallySubordinateTo_list(L1,L2) :-
   list(L1),
   list(L2),
   L1 = [H1| _],
   L2 = [H2| _],
   id__lt_id(H1,H2),
   !.
list__isLexicographicallySubordinateTo_list(L1,L2) :-
   list(L1),
   list(L2),
   L1 = [H| T1],
   L2 = [H| T2],
   id__lt_id(T1,T2).


list__beginsWith_chars([W| _],W) :- 
   chars(W),
   !.                                      % Solution is sufficient.
list__beginsWith_chars(X,Y) :-
   var(X),
   o_exceptionMsg('list__beginsWith_chars',
                [X,Y],
                'is queried with unbound first arg'),
   fail.


list__endsWith_chars([W],W) :- 
   chars(W),
   !.                                      % Solution is sufficient.
list__endsWith_chars([_| L],W) :-
   nonvar(L),
   list__endsWith_chars(L,W),
   !.                                      % Solution is sufficient.
list__endsWith_chars(X,Y) :-
   var(X),
   o_exceptionMsg('list__endsWith_chars',
                [X,Y],
                'is queried with unbound first arg'),
   fail.



% :- consult('flatlist.pro').
% :- consult('list.pro').
% :- consult('chars.pro').

