% FLATLIST.PRO: Standard theory about flat lists.
% Author: Frans Meijs, 1993

% A flat list is a list which contains no other lists.



flatList__beginsWith_flatList__of_length(_,[],0) :-
   !.
flatList__beginsWith_flatList__of_length(L1,L2,N) :-
   nonvar(L1), nonvar(N),
   N > 0,
   L1 = [L3| L4],
   N2 is N - 1,
   flatList__beginsWith_flatList__of_length(L4,L5,N2),
   list__appendedWith_list__is_list([L3],L5,L2),
   !.
flatList__beginsWith_flatList__of_length(X,Y,Z) :-
   ~( list(X)),
   o_exceptionMsg('flatList__beginsWith_flatList__of_length',
                [X,Y,Z],
                'is queried with wrong first arg'),
   fail.
flatList__beginsWith_flatList__of_length(X,Y,Z) :-
   var(Z),
   o_exceptionMsg('flatList__beginsWith_flatList__of_length',
                [X,Y,Z],
                'is queried with wrong third arg'),
   fail.
                                   % Length is counted in terms of
                                   % member terms.
 


% flatList__isSublist_of_flatList(L1, L2) :-
%   list__appendedWith_list__is_list(_, L1, L3),
%   list__appendedWith_list__is_list(L3, _, L2),
%   groundTerm(L3).
                % Extremely inefficient; Any other solution is better!


flatList__has_vars(FL, [Var]) :-
   nonvar(FL),
   FL = [Var],
   var(Var),
   !.                                          % Solution is sufficient.
flatList__has_vars(FL, [Var]) :-
   nonvar(FL),
   FL = [Var| [Nonvar]],
   var(Var),
   nonvar(Nonvar),
   !.                                          % Solution is sufficient.
flatList__has_vars(FL, [Var| List_of_var]) :-
   nonvar(FL),
   FL = [Var| FlatList],
   var(Var),
   flatList__has_vars(FlatList, List_of_var),
   !.                                          % Solution is sufficient.
flatList__has_vars([Nonvar| FlatList], List_of_var) :-
   nonvar(Nonvar),
   flatList__has_vars(FlatList, List_of_var),
   !.                                          % Solution is sufficient.
flatList__has_vars(FL, VFL) :-
   var(FL),
   o_exceptionMsg('flatList__has_vars',
                [FL, VFL],
                'is queried with unbound first arg'),
   fail.



flatListOfChars([]) :-
   !.                                          % Solution is sufficient.
flatListOfChars([C| L]) :-
   chars(C),
   flatListOfChars(L).




% :-consult('list.pro').
