% INT.PRO: Theory about integer numerals.
% This file has to be consulted by Prolog programs.


odd(X) :-
   integer(X),
   int__absIs_int(X,X2),
   R is X2 mod 2,
   R = 1.


int__plus_int__is_int(N1,N2,N3) :-
   integer(N1),
   integer(N2),
   N3 is N1 + N2,
   !.                                        % Solution is sufficient.
int__plus_int__is_int(N1,N2,N3) :-
   o_exceptionMsg('int__plus_int__is_int',
                [N1,N2,N3],
                'is queried with wrong args'),
   fail.
      % This clause is used for making IPL theories verifier-friendly.
      % It also fails instead of halting interpretation.


int__minus_int__is_int(N1,N2,N3) :-
   integer(N1),
   integer(N2),
   N3 is N1 - N2,
   !.                                       % Solution is sufficient.
int__minus_int__is_int(N1,N2,N3) :-
   o_exceptionMsg('int__minus_int__is_int',
                [N1,N2,N3],
                'is queried with wrong args'),
   fail.
      % This clause is used for making IPL theories verifier-friendly.
      % It also fails instead of halting interpretation.


int__times_int__is_int(N1,N2,N3) :-
   integer(N1),
   integer(N2),
   N3 is N1 * N2,
   !.                                       % Solution is sufficient.
int__times_int__is_int(N1,N2,N3) :-
   o_exceptionMsg('int__times_int__is_int',
                [N1,N2,N3],
                'is queried with wrong args'),
   fail.
      % This clause is used for making IPL theories verifier-friendly.
      % It also fails instead of halting interpretation.


int__div_int__is_int(N,D,Q) :-
   integer(N),
   integer(D),
   int__div_int__is_int__aux(N,D,Q),
   !.                                       % Solution is sufficient.
int__div_int__is_int(N1,N2,N3) :-
   o_exceptionMsg('int__div_int__is_int',
                [N1,N2,N3],
                'is queried with wrong args'),
   fail.
      % This clause is used for making IPL theories verifier-friendly.
      % It also fails instead of halting interpretation.



int__max_int__is_int( X, Y, X ) :- integer(X), X >= Y, !.
int__max_int__is_int( X, Y, Y ) :- integer(X), integer(Y).

int__min_int__is_int( X, Y, Y ) :- integer(X), X >= Y, !.
int__min_int__is_int( X, Y, X ) :- integer(X), integer(Y).


int__and_int__have_gcd(N,  N,  N) :- integer(N).
int__and_int__have_gcd(N1, N2, D) :-
   int__absIs_int(N1, N1a),
   int__absIs_int(N2, N2a),
   N1a > N2a,
   N3 is N1a - N2a,
   int__and_int__have_gcd(N3, N2a, D).
int__and_int__have_gcd(N1, N2, D) :-
   int__absIs_int(N1, N1a),
   int__absIs_int(N2, N2a),
   N1a < N2a,
   int__and_int__have_gcd(N2a, N1a, D).


int__absIs_int( X, X ) :- integer(X), X >= 0, !.
int__absIs_int( X, Y ) :- integer(X), Y is -X.

