% NUMLIST.PRO: Standard theory about lists of numerals.
% Author: Frans Meijs, 1995.


numList__sortsInto_numList([],[]).
           % Same method as in list__sortsLexicographicallyInto_list.
numList__sortsInto_numList([H|T],Result) :-
   numeral__splits_numList_into_minorNumList__and_majorNumList(
                                                         H,T,U1,U2),
                                     % U1 = list of entries <= H
                                     % U2 = list of entries > H
   numList__sortsInto_numList(U1,V1),
   numList__sortsInto_numList(U2,V2),
   list__appendedWith_list__is_list(V1,[H|V2],Result).


% numeral__splits_numList__into_minorNumList__and_majorNumList(
%                                              arg1,arg2,arg3,arg4).
% Arg 3 consists of all elements from arg 2 <= arg 1; arg 4 contains
% the remaining elements.

numeral__splits_numList__into_minorNumList__and_majorNumList(_,[],[],[]).
numeral__splits_numList__into_minorNumList__and_majorNumList(
                                                      N,[H1|T1],[H1|LT],GE) :-
   int__le_int(H1,N),
   numeral__splits_numList__into_minorNumList__and_majorNumList(N,T1,LT,GE).
numeral__splits_numList__into_minorNumList__and_majorNumList(
                                                      N,[H1|T1],LT,[H1|GE]) :-
   H1 > N,
   numeral__splits_numList_into_minorNumList__and_majorNumList(N,T1,LT,GE).


numList__has_mean(L,M) :-
   list__has_length(L,N),
   N > 0,
   numList__has_sum(L,S),
   M is S / N.


numList__has_sum(L,0) :-
   L == [],                              % L should not unify with [].
   !.                                      % Solution is sufficient.
numList__has_sum(L,S) :-
   nonvar(L),
   L = [S],
   number(S),
   !.                                      % Solution is sufficient.
numList__has_sum(L,S) :-
   nonvar(L),
   L = [H|L2],
   number(H),
   numList__has_sum(L2,S2),
   S is H + S2,
   !.                                      % Solution is sufficient.
numList__has_sum(L,S) :-
   nonvar(L),
   L = [H|_],
   ~( number(H) ),
   o_exceptionMsg('numList__has_sum',
                [L,S],
                'is queried with non-numeric first arg'),
   fail.
numList__has_sum(L,S) :-
   var(L),
   o_exceptionMsg('numList__has_sum',
                [L,S],
                'is queried with unbound first arg'),
   fail.




