% OSIW.PRO: Standard theory about object identifier words.
% (An object specification is a specification instantiated on the  
%  output channel).

% Words which are Prolog-type lists, and words and list elements which
% are character strings (atoms) containing space characters, are 
% converted to flat character strings without space characters. 


o_apostrophedObjectIdWord(I) :-
   put(39),
   o_IDword(I),
   put(39).


o_IDword(I) :-
   term__eq_objectIdWord(I, I_ID),
   o_chars(I_ID),
   !.                                        % Solution is sufficient. 
o_IDword(_) :-
   o_char('?').                                    % Illegal arg.


term__eq_objectIdWord(I, I_ID) :-
   chars(I),
   chars__eq_objectIdWord(I, I_ID),
   !.                                   % Solution is sufficient. 
term__eq_objectIdWord(I, I_ID) :-
   list__eq_flatList(I, SI),
   listOfChars__eq_objectIdWord(SI, I_ID).


listOfChars__eq_objectIdWord([C], W) :-
   chars(C),
   chars__eq_objectIdWord(C, W),                 % No upcasing.
   !.                                   % Solution is sufficient.
listOfChars__eq_objectIdWord([C| T], W) :-
   chars(C),
   chars__eq_objectIdWord(C, W2),                % No upcasing.
   listOfChars__eq_objectIdWord__v2(T, W3),
   chars__appendedWith_chars__is_chars(W2, W3, W).


listOfChars__eq_objectIdWord__v2([C], W) :-
   chars(C),
   chars__eq_objectIdWord__v2(C, W),            % With upcasing.
   !.                                        % Solution is sufficient.
listOfChars__eq_objectIdWord__v2([C| T], W) :-
   chars__eq_objectIdWord__v2(C, W2),
   listOfChars__eq_objectIdWord__v2(T, W3),
   chars__appendedWith_chars__is_chars(W2, W3, W).


chars__eq_objectIdWord(C, I) :-
   chars(C), var(I),
   chars__eq_objectIdWordAsciiList(C, List2),
   chars__eq_asciiNumeralList(I, List2),
   !.                                        % Solution is sufficient.
chars__eq_objectIdWord(X, Y) :-
   o_exceptionMsg('chars__eq_objectIdWord',
                [X, Y],
                'is queried with wrong args'),
   fail.


chars__eq_objectIdWord__v2(C, I) :-
   chars(C), var(I),
   chars__eq_objectIdWordAsciiList(C, List2),
   asciiNumeralList__withHeadUpcasedIs_asciiNumeralList(List2, 
                                                               List3),
   chars__eq_asciiNumeralList(I, List3),
   !.                                   % Solution is sufficient.
chars__eq_objectIdWord__v2(X, Y) :-
   o_exceptionMsg('chars__eq_objectIdWord__v2',
                [X, Y],
                'is queried with wrong args'),
   fail.


chars__eq_objectIdWordAsciiList(C, L) :-
   chars(C), var(L),
   chars__eq_asciiNumeralList(C, L2),
   asciiNumeralList__withSpacesReplacedByUpcaseIs_asciiNumeralList(
                                                            L2, L),
   !.                                   % Solution is sufficient.
chars__eq_objectIdWordAsciiList(X, Y) :-
   o_exceptionMsg('chars__eq_objectIdWordAsciiList',
                [X, Y],
                'is queried with wrong args'),
   fail.


%:- consult('ascii.pro').
%:- consult('list.pro').
%:- consult('chars.pro').
