% RAT.PRO: Standard theory about rational numerals.
% This file has to be consulted by Prolog programs.


rat([N, D]) :- integer(N), integer(D), D \== 0.


rat__plus_rat__is_rat([N1, D1], [N2, D2], [Ns, Ds]) :-
   rat([N1, D1]),              % Appears in the righthand side of "is".
   rat([N2, D2]),              % Appears in the righthand side of "is".
   N3 is N1 * D2 + N2 * D1,
   D3 is D1 * D2,
   rat__normalizesInto_rat([N3, D3], [Ns, Ds]).


rat__minus_rat__is_rat([N1, D1], [N2, D2], [Ns, Ds]) :-
   rat([N1, D1]),              % Appears in the righthand side of "is".
   rat([N2, D2]),              % Appears in the righthand side of "is".
   N3 is N1 * D2 - N2 * D1,
   D3 is D1 * D2,
   rat__normalizesInto_rat([N3, D3], [Ns, Ds]).


rat__times_rat__is_rat([N1, D1], [N2, D2], [Np, Dp]) :-
   rat([N1, D1]),              % Appears in the righthand side of "is".
   rat([N2, D2]),              % Appears in the righthand side of "is".
   N3 is N1 * N2,
   D3 is D1 * D2,
   rat__normalizesInto_rat([N3, D3], [Np, Dp]).


rat__dividedBy__rat__is_rat([N1, D1], [N2, D2], [Nq, Dq]) :-
   rat([N1, D1]),              % Appears in the righthand side of "is".
   rat([N2, D2]),              % Appears in the righthand side of "is".
   N3 is N1 * D2,
   D3 is D1 * N2,
   rat__normalizesInto_rat([N3, D3], [Nq, Dq]).


rat__normalizesInto_rat([N, D], [Nn, Dn]) :-
   rat([N, D]),                % Appears in the righthand side of "is".
   int__and_int__have_gcd(N, D, GCD),
   int__div_int__is_int(N, GCD, N2),
   int__div_int__is_int(D, GCD, D2),
   rat__normalizes2Into_rat([N2, D2], [Nn, Dn]).

rat__normalizes2Into_rat([N, D], [N, D]) :-
   D > 0, !.
rat__normalizes2Into_rat([N, D], [Nn, Dn]) :-
   D < 0,
   Nn is -N,
   Dn is -D.


rat__lt_rat([N1, D1], [N2, D2]) :-
   rat__subtractedBy_rat__is_rat([N1, D1], [N2, D2], [Ns, Ds]),
   Ns < 0.

rat__le_rat([N1, D1], [N2, D2]) :-
   rat__subtractedBy_rat__is_rat([N1, D1], [N2, D2], [Ns, Ds]),
   int__le_int(Ns, 0).

