% OUTMATRI.PRO: Standard theory about output matrices.


% A cell of a matrix can be specified by a set of propositions of the 
% form:

%                     matrixCell__represents_concept(CE,S).
%                     cell__e_row(CE,R).
%                     row__e_matrix(R,M).

% For an output matrix,the ordering of the cells and rows can be 
% specified by a set of propositions of the form:

%                ordering__isSubordinateTo_concept(O,C).
%                dimension__e_numberedOrdering(D,O).

% For cells which are not specified,a "." will be written in the output.


% An output matrix with a row per record can be instantiated by a query 
% of the form:

%                     o_matrix(M). 


%-----------------------------------------------------------------------
% When the addressing scheme of theory MATRIX.PRO is used,a heading of 
% the matrix can be instantiated by a query of the form:


%                  o_matrixHeading(M).


% See the example in file "outmatri.tst".


%-----------------------------------------------------------------------
o_matrixHeading(['m',R,C]) :-
   o_spaces(9),
   o_matrixHeading_2(['m',R,C]),o_eor,
   o_eor.   

o_matrixHeading_2(['m',R,C]) :-
   dimension__e_numberedOrdering(1,['r',R,C]),
   o_spaces(2),
   o_chars(C).  

o_matrixHeading_2(['m',R,C]) :-
   ordering__isSubordinateTo_concept(['r',R,C2],['c',R,C]),
   o_matrixHeading_2(['m',R,C2]),
   o_spaces(2),
   o_chars(C).        


o_cell__e_row(CE,R) :-
   dimension__e_numberedOrdering(1,R),
   cell__e_row(CE,R),
   matrixCell__represents_concept(CE,I),
   o_spaces(2),
   o_chars(I).

o_cell__e_row(CE,R) :-
   dimension__e_numberedOrdering(1,R),
   cell__e_row(CE,R),
   ~( matrixCell__represents_concept(CE,_) ),
   o_spaces(2),
   o_chars('.').

o_cell__e_row(CE,R) :-
   ~( dimension__e_numberedOrdering(1,R) ),
   cell__e_row(CE,R),
   matrixCell__represents_concept(CE,I),
   ordering__isSubordinateTo_concept(R2,CE),
   o_cell__e_row(_,R2),
   o_spaces(2),
   o_chars(I).

o_cell__e_row(CE,R) :-
   ~( dimension__e_numberedOrdering(1,R) ),
   cell__e_row(CE,R),
   ~( matrixCell__represents_concept(CE,_) ),
   ordering__isSubordinateTo_concept(R2,CE),
   o_cell__e_row(_,R2),
   o_spaces(2),
   o_chars('.').


o_row__e_matrix(R,M) :-
   dimension__e_numberedOrdering(1,M),
   row__e_matrix(R,M),
   o_chars(R),
   o_spaces(2),
   o_cell__e_row(_,R),
   o_eor.

o_row__e_matrix(R,M) :-
   ~( dimension__e_numberedOrdering(1,M) ),
   row__e_matrix(R,M),
   ordering__isSubordinateTo_concept(M2,R),
   o_row__e_matrix(_,M2),
   o_chars(R),
   o_spaces(2),
   o_cell__e_row(_,R),
   o_eor. 

o_matrix(M) :- 
   o_row__e_matrix(_,M),
   fail.               % The interpreter is forced to find all rows.


