% DSM.PRO: Abstractions about Discrete State Machines;
% Definition with abstraction from the individual state variables,
% and trace connection as described by pre- and postconditions.
% Author: Frans Meijs, nov 1998


behaviouralUnit(DSM) :- dsm(DSM).


stateVariable(SV) :- ssmsVariable(SV).
                     % "ssms" stands for "Subsort of machine states"
                     % "ssmsv" stands for "SSMS variable"



mappingClass(PTC) :- conditionalTc(PTC).
                         % Time domain population.



mappingClass__e_bu(TC,DSM) :-
   tc__e_dsm(TC,DSM).


subSortOfMachineStates__i_ssmsv(SSMS,MSVV) :-
   var(SSMS), nonvar(MSVV),
   tc__mapsFrom_ssmsv(TC,MSVV),
   tc__mapsTo_ssmsv(TC,MSVV),
   tc__mapsFrom_ssms(TC,SSMS2),
   subSortOfMachineStates__isExplicitlyInstanceOf_ssmsv(SSMS2,MSVV),
   tc__mapsFrom_sev(TC,SEV),    % Weakest precond, so not msgv.
   state__isExplicitlyInstanceOf_stateVariable('TRUE',SEV),
   state__isNextInstanceOf_stateVariable('FALSE',SEV),
                                                % Note 3.
   tc__mapsTo_setOfMsgv(TC,_),
              % update of the output MSGV-s,if any (Side effect).
   tc__mapsTo_ssms(TC,SSMS),
   subSortOfMachineStates__isNextInstanceOf_ssmsv(SSMS,MSVV).
              % update of the spec of the machine state variables,
              % and of the triggering SEV.
subSortOfMachineStates__i_ssmsv(SSMS,MSVV) :-
   var(SSMS), nonvar(MSVV),
   stateVariable__isPrivateElementOf_bu(MSVV,BU),
   stateVariable__isInputElementOf_bu(SEV,BU),
   signallingEventVariable(SEV),
   state__isExplicitlyInstanceOf_stateVariable('TRUE',SEV),
   state__isNextInstanceOf_stateVariable('FALSE',SEV),
               % Note 4.
   subSortOfMachineStates__isExplicitlyInstanceOf_ssmsv(SSMS,MSVV).

         % (This theory represents the operational semantics of DSM-s)


class__isAssociatedWith_set(C,S) :-
   tc__mapsFrom_ssms(C,S).
class__isAssociatedWith_set(C,S) :-
   tc__mapsStronglyFrom_ssms(C,S).
class__isAssociatedWith_set(C,S) :-
   tc__mapsWeaklyFrom_ssms(C,S).
class__isAssociatedWith_set(C,S) :-
   tc__mapsTo_ssms(C,S).


msgv__isInputElementOf_su(MSGV,SU) :-
   nonvar(MSGV),
   dsmId__eq_enablingMsgvId(DSM,MSGV),
   exists( tc__enables_dsm(_,DSM) ),
   behaviouralUnit__e_su(DSM,SU).
msgv__isInputElementOf_su(MSGV,SU) :-
   var(MSGV), nonvar(SU),
   behaviouralUnit__e_su(DSM,SU),
   exists( tc__enables_dsm(_,DSM) ),
   dsmId__eq_enablingMsgvId(DSM,MSGV).
msgv__isInputElementOf_su(MSGV,SU) :-
   nonvar(MSGV),
   dsmId__eq_disablingMsgvId(DSM,MSGV),
   exists( tc__disables_dsm(_,DSM) ),
   behaviouralUnit__e_su(DSM,SU).
msgv__isInputElementOf_su(MSGV,SU) :-
   var(MSGV), nonvar(SU),
   behaviouralUnit__e_su(DSM,SU),
   exists( tc__disables_dsm(_,DSM) ),
   dsmId__eq_disablingMsgvId(DSM,MSGV).
  

tc__mapsFrom_ssmsv(TC,MSVV) :-
   nonvar(TC),
   tc__e_dsm(TC,DSM),
   dsmId__eq_ssmsvId(DSM,MSVV).
tc__mapsFrom_ssmsv(TC,MSVV) :-
   var(TC),nonvar(MSVV),
   dsmId__eq_ssmsvId(DSM,MSVV),
   tc__e_dsm(TC,DSM).


tc__mapsTo_ssmsv(TC,MSVV) :-
   nonvar(TC),
   tc__e_dsm(TC,DSM),
   dsmId__eq_ssmsvId(DSM,MSVV).
tc__mapsTo_ssmsv(TC,MSVV) :-
   var(TC),nonvar(MSVV),
   dsmId__eq_ssmsvId(DSM,MSVV),
   tc__e_dsm(TC,DSM).


tc__e_dsm(TC,DSM) :-
   conditionalTc__e_dsm(TC,DSM),
   conditionalTc__mapsFrom_precondSsms(TC,SSMS),
   exists( conditionalTc__mapsTo_postcondSsms(_,SSMS) ).
tc__e_dsm(TC,DSM) :-
   nonvar(TC),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   conditionalTc__e_dsm(CTC,DSM).
tc__e_dsm(TC,DSM) :-
   var(TC), nonvar(DSM),
   conditionalTc__e_dsm(CTC,DSM),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS).
          % Synthetic transition class due to trace connection.


tc__mapsStronglyFrom_ssms(TC,SSMS) :-
   tc__mapsFrom_ssms(TC,SSMS),
   exists( tc__mapsTo_ssms(_,SSMS) ).
tc__mapsStronglyFrom_ssms(PTC,POCSSMS) :-
   nonvar(PTC),
   tc__mapsFrom_ssms(PTC,PRCSSMS),
   sort__e_sort(POCSSMS,PRCSSMS),
   exists( tc__mapsTo_ssms(_,POCSSMS) ).
                                 % DSM state supersorts are excluded.
tc__mapsStronglyFrom_ssms(PTC,POCSSMS) :-
   var(PTC), nonvar(POCSSMS),
   exists( tc__mapsTo_ssms(_,POCSSMS) ),
                                 % DSM state supersorts are excluded.
   sort__e_sort(POCSSMS,PRCSSMS),
   tc__mapsFrom_ssms(PTC,PRCSSMS).


tc__mapsWeaklyFrom_ssms(TC,POCSSMS) :-
   nonvar(TC),
   tc__mapsFrom_ssms(TC,PRCSSMS),
   sort__intersects_sort(POCSSMS,PRCSSMS),
                     % ~( sort__e_sort(POCSSMS,PRCSSMS) ) is not necessary.
   exists( tc__mapsTo_ssms(_,POCSSMS) ).
                                 % DSM state supersorts are excluded.
tc__mapsWeaklyFrom_ssms(TC,POCSSMS) :-
   var(TC), nonvar(POCSSMS),
   exists( tc__mapsTo_ssms(_,POCSSMS) ),
                                 % DSM state supersorts are excluded.
   sort__intersects_sort(POCSSMS,PRCSSMS),
   tc__mapsFrom_ssms(TC,PRCSSMS).


inSevOutMsgvToSsmsTriplet__mapsFrom_ssms(ISOSTMST,SSMS) :-     % Aux def.
   nonvar(SSMS),
   tc__mapsFrom_ssms(TC,SSMS),
   tc__mapsTo_setOfMsgv(TC,MSGV2_LIST),
   tc__mapsFrom_sev(TC,SEV1),    % Weakest precond, so not msgv.
   tc__mapsTo_ssms(TC,SSMS2),
   ISOSTMST = [SEV1,MSGV2_LIST,SSMS2].


tc__mapsFrom_ssms(TC,SSMS) :-
   conditionalTc__mapsFrom_precondSsms(TC,SSMS),
   exists( conditionalTc__mapsTo_postcondSsms(_,SSMS) ).
tc__mapsFrom_ssms(TC,SSMS) :-
   nonvar(TC),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,SSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,SSMS).
               % Synthetic transition class due to trace connection.
tc__mapsFrom_ssms(TC,SSMS) :-
   var(TC), nonvar(SSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,SSMS),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,SSMS).


tc__mapsTo_ssms(TC,SSMS) :-
   conditionalTc__mapsTo_postcondSsms(TC,SSMS).
tc__mapsTo_ssms(TC,SSMS) :-
   nonvar(TC),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   conditionalTc__mapsTo_postcondSsms(CTC,SSMS).
               % Synthetic transition class due to trace connection.
tc__mapsTo_ssms(TC,SSMS) :-
   var(TC), nonvar(SSMS),
   conditionalTc__mapsTo_postcondSsms(CTC,SSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS).



elemStateMappingClass__mapsFrom_stateVariable(TC,SEV) :-
   tc__mapsFrom_sev(TC,SEV).


tc__mapsFrom_sev(TC,SEV) :-
   tc__mapsExplicitlyFrom_sev(TC,SEV),
   ~( class__isExplicitlySubclassOf_class(_,SEV) ).
tc__mapsFrom_sev(TC,SEV) :- 
   nonvar(TC), var(SEV),
   tc__mapsExplicitlyFrom_sev(TC,SEV2),
   class__isElementarySubclassOf_class(SEV,SEV2).
tc__mapsFrom_sev(TC,SEV) :- 
   nonvar(SEV),
   class__isElementarySubclassOf_class(SEV,SEV2),
   tc__mapsExplicitlyFrom_sev(TC,SEV2).
                              % Shorthand notation for SEV-s.

tc__mapsExplicitlyFrom_sev(TC,SEV) :-
   tc__mapsFrom_msgv(TC,MSGV),
   sev__e_msgv(SEV,MSGV).
   

tc__mapsFrom_msgv(TC,MV) :-
   conditionalTc__mapsFrom_msgv(TC,MV),
   conditionalTc__mapsFrom_precondSsms(TC,SSMS),
   exists( conditionalTc__mapsTo_postcondSsms(_,SSMS) ).
tc__mapsFrom_msgv(TC,MV) :-
   nonvar(TC),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   conditionalTc__mapsFrom_msgv(CTC,MV).
               % Synthetic transition class due to trace connection.
tc__mapsFrom_msgv(TC,MV) :-
   var(TC), nonvar(MV),
   conditionalTc__mapsFrom_msgv(CTC,MV),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS). 


tc__mapsTo_sev(TC,SEV) :-
   tc__mapsTo_msgv(TC,MSGV),
   sev__e_msgv(SEV,MSGV).
   

tc__mapsTo_setOfMsgv(TC,SEV_LIST) :-
   nonvar(TC),
   setofall(SEV,tc__mapsTo_msgv(TC,SEV),SEV_LIST).


tc__mapsTo_msgv(TC,MSGV) :-
   conditionalTc__mapsTo_msgv(TC,MSGV),
   conditionalTc__mapsFrom_precondSsms(TC,SSMS),
   exists( conditionalTc__mapsTo_postcondSsms(_,SSMS) ).
tc__mapsTo_msgv(TC,MSGV) :-
   nonvar(TC),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   conditionalTc__mapsTo_msgv(CTC,MSGV).
               % Synthetic transition class due to trace connection.
tc__mapsTo_msgv(TC,MSGV) :-
   var(TC), nonvar(MSGV),
   conditionalTc__mapsTo_msgv(CTC,MSGV),
   conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS).
tc__mapsTo_msgv(TC,MSGV) :-
   nonvar(TC),
   tc__enables_dsm(TC,DSM),
   dsmId__eq_enablingMsgvId(DSM,MSGV).
tc__mapsTo_msgv(TC,MSGV) :-
   var(TC), nonvar(MSGV),
   dsmId__eq_enablingMsgvId(DSM,MSGV),
   tc__enables_dsm(TC,DSM).
tc__mapsTo_msgv(TC,MSGV) :-
   nonvar(TC),
   tc__disables_dsm(TC,DSM),
   dsmId__eq_disablingMsgvId(DSM,MSGV).
tc__mapsTo_msgv(TC,MSGV) :-
   var(TC), nonvar(MSGV),
   dsmId__eq_disablingMsgvId(DSM,MSGV),
   tc__disables_dsm(TC,DSM).


tc__enables_dsm(TC,BU) :-
   conditionalTc__enables_dsm(TC,BU),
   conditionalTc__mapsFrom_precondSsms(TC,SSMS),
   exists( conditionalTc__mapsTo_postcondSsms(_,SSMS) ).
tc__enables_dsm(TC,BU) :-
   nonvar(TC),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,SSMS),
   conditionalTc__enables_dsm(CTC,BU),
   conditionalTc__mapsFrom_postcondSsms(CTC,SSMS).
               % Synthetic transition class due to trace connection.
tc__enables_dsm(TC,BU) :-
   var(TC), nonvar(SSMS),
   conditionalTc__mapsFrom_postcondSsms(CTC,SSMS),
   conditionalTc__enables_dsm(CTC,BU),
   tc__eq_conditionalTc__with_postcondSsms(TC,CTC,SSMS).
   

conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS) :-
   nonvar(CTC),
   conditionalTc__mapsFrom_precondSsms(CTC,PRCSSMS),
   sort__intersects_sort__v2(POCSSMS,PRCSSMS),  
   exists( conditionalTc__mapsTo_postcondSsms(_,POCSSMS) ).
conditionalTc__mapsFrom_postcondSsms(CTC,POCSSMS) :-
   var(CTC), nonvar(POCSSMS),
   exists( conditionalTc__mapsTo_postcondSsms(_,POCSSMS) ),
   sort__intersects_sort__v2(POCSSMS,PRCSSMS),
   conditionalTc__mapsFrom_precondSsms(CTC,PRCSSMS).
                  % The theory about trace connections.


sort__intersects_sort__v2(S1,S2) :-
   sort__intersects_sort(S1,S2).
sort__intersects_sort__v2(S1,S2) :-
   sort__isExplicitlyElementOf_sort(S1,S2).
                           % sort__e_sort is not necessary. 


sort(S) :-
   dsm(DSM),
   ~( fsm(DSM) ),
   dsm__isAssociatedWith_sms(DSM,S).
                     % "sms" stands for "Sort of machine states"
             

sort__isExplicitlyElementOf_sort(S1,S2) :-
   nonvar(S1),
   dsm__isAssociatedWith_sms(DSM,S1),
   ~( fsm(DSM) ),
   S2 = ['SortOf','entities'].
sort__isExplicitlyElementOf_sort(S1,S2) :-
   var(S1),nonvar(S2),
   S2 = ['SortOf','entities'],
   dsm(DSM),
   ~( fsm(DSM) ),
   dsm__isAssociatedWith_sms(DSM,S1).
sort__isExplicitlyElementOf_sort(POCS,PRCS) :-
   nonvar(POCS),
   conditionalTc__mapsTo_postcondSsms(TC1,POCS),
   conditionalTc__e_dsm(TC1,DSM),
   conditionalTc__e_dsm(TC2,DSM),
   TC1 \= TC2,
   conditionalTc__mapsFrom_precondSsms(TC2,PRCS),
   condition__of_dsm__represents_ssms('TRUE',DSM,PRCS).
                % Implication for precond "TRUE".
sort__isExplicitlyElementOf_sort(POCS,PRCS) :-
   var(POCS), nonvar(PRCS),
   conditionalTc__mapsFrom_precondSsms(TC2,PRCS),
   conditionalTc__e_dsm(TC2,DSM),
   condition__of_dsm__represents_ssms('TRUE',DSM,PRCS),
   conditionalTc__e_dsm(TC1,DSM),
   TC1 \= TC2,
   conditionalTc__mapsTo_postcondSsms(TC1,POCS).


postCondSsms__isAssociatedWith_dsm(SMS,DSM) :-
   conditionalTc__e_dsm(TC,DSM),
   exists( conditionalTc__mapsTo_postcondSsms(TC,SMS) ).


dsm__isAssociatedWith_sms(DSM,SMS) :-
   nonvar(DSM),
   stateVariable__isPrivateElementOf_bu(SV,DSM),
   ssmsVariable(SV),
   stateVariable__isAssociatedWith_sort(SV,SMS).
dsm__isAssociatedWith_sms(DSM,SMS) :-
   var(DSM),nonvar(SMS),
   stateVariable__isAssociatedWith_sort(SV,SMS),
   ssmsVariable(SV),
   stateVariable__isPrivateElementOf_bu(SV,DSM).
         % Ref. to tc__e_dsm/2 would cause a circular implication here.


ssmsVariable(SSMSV) :-
   nonvar(SSMSV),
   dsmId__eq_ssmsvId(DSM,SSMSV),
   exists( dsm(DSM) ).
ssmsVariable(SSMSV) :-
   var(SSMSV),
   dsm(DSM),
   dsmId__eq_ssmsvId(DSM,SSMSV).


stateVariable__isPrivateElementOf_bu(SV,DSM) :-  
   nonvar(SV),
   dsmId__eq_ssmsvId(DSM,SV),
   exists( dsm(DSM) ).
stateVariable__isPrivateElementOf_bu(SV,DSM) :-
   var(SV),
   dsm(DSM),
   dsmId__eq_ssmsvId(DSM,SV).


tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS) :-   % Aux def.
   nonvar(TC),
   tcId__eq_conditionalTcId__with_postcondId(TC,CTC,POC),
   conditionalTc__e_dsm(CTC,DSM),
   condition__of_dsm__represents_ssms(POC,DSM,POCSSMS).  
tc__eq_conditionalTc__with_postcondSsms(TC,CTC,POCSSMS) :-
   var(TC),nonvar(CTC),nonvar(POCSSMS),
   conditionalTc__e_dsm(CTC,DSM),
   condition__of_dsm__represents_ssms(POC,DSM,POCSSMS), 
   tcId__eq_conditionalTcId__with_postcondId(TC,CTC,POC).



elemState__e_entity(P,DSM) :-
   executionPriority__e_dsm(P,DSM).


executionPriority__e_dsm(P,DSM) :-
   nonvar(DSM),
   exists( dsm(DSM) ),
   behaviouralUnit__e_su(DSM,SU),
   executionPriority__e_su(P,SU).




%:-consult('dsm.prq').
%:-consult('dsm.prc').


% Notes:

% 1. For preventing duplicate instances caused by multiple inheritance
%    of subclasses.
% 3. This is the characteristic of an SEV; it is set to "FALSE"
%    once it has implied a transition.
% 4. Cancellation of input SEV-s which did not imply a machine
%    state variable transition (also in case of no triggering SEV). 


